﻿using MindFusion.Charting;
using MindFusion.Drawing;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace BubbleChart
{
	// Learn more about making custom code visible in the Xamarin.Forms previewer
	// by visiting https://aka.ms/xamarinforms-previewer
	[DesignTimeVisible(false)]
	public partial class MainPage : ContentPage
	{
		public MainPage()
		{
			InitializeComponent();
            // create sample data
            bubbleChart.Series = GetSeriesCollection();

            // axis titles and ranges
            bubbleChart.XAxis.Title = "Average relative annual growth (%)";
            bubbleChart.XAxis.MinValue = -1;
            bubbleChart.XAxis.MaxValue = 1;
            bubbleChart.YAxis.Title = "July 1, 2015 projection";
            bubbleChart.YAxis.MinValue = 0;
            bubbleChart.YAxis.MaxValue = 100;

            // background appearance
            bubbleChart.ShowZoomWidgets = true;
            bubbleChart.GridType = GridType.Vertical;
            bubbleChart.BackgroundColor = Color.Black;
            bubbleChart.Theme.GridColor1 = Color.FromRgba(11, 11, 11, 100);
            bubbleChart.Theme.GridColor2 = Color.FromRgba(33, 33, 33, 200);
            bubbleChart.Theme.LegendBackground = new SolidBrush(Color.Black);
			bubbleChart.Theme.PlotBorderStroke = new SolidBrush(Color.White);
			bubbleChart.Theme.PlotBorderStrokeThickness = 2.0;
			bubbleChart.Theme.DataLabelsFontSize = 12;
			bubbleChart.Theme.AxisLabelsFontSize = 12;
			bubbleChart.Theme.AxisTitleFontSize = 14;

			// series colors
			bubbleChart.Theme.CommonSeriesFills = new List<Brush>
            {
                new LinearGradientBrush(Color.Transparent, Colors.Orange, 90),
                new LinearGradientBrush(Color.Transparent, Colors.SkyBlue, 90)
            };
            bubbleChart.Theme.UniformSeriesStroke = bubbleChart.Theme.HighlightStroke =
                bubbleChart.Theme.DataLabelsBrush = bubbleChart.Theme.LegendTitleBrush =
                bubbleChart.Theme.LegendBorderStroke = bubbleChart.Theme.AxisLabelsBrush =
                bubbleChart.Theme.AxisTitleBrush = bubbleChart.Theme.AxisStroke = Brushes.White;

            bubbleChart.Theme.HighlightStrokeDashStyle = DashStyle.Dot;

            SetupControls();
        }

        void SetupControls()
        {
            cbShowScatter.IsToggled = bubbleChart.ShowScatter;
            cbShowLabels.IsToggled = true;
            cbLegend.IsToggled = true;
        }

        ObservableCollection<Series> GetSeriesCollection()
        {
            // bubble chart requires three dimensional data;
            // two dimensions for position and one for size
            var series3D1 = new PointSeries3D(
                new List<Point3D>
                {
                    new Point3D(0.32, 81, 95),
                    new Point3D(0.39, 66, 78),
                    new Point3D(0.75, 65, 76),
                    new Point3D(0.49, 60, 71)
                },
                new List<string> { "Germany", "France", "UK", "Italy" });
            series3D1.Title = ">50 000 000";

            var series3D2 = new PointSeries3D(
                new List<Point3D>
                {
                    new Point3D(-0.28, 46, 54),
                    new Point3D(-0.32, 42, 50),
                    new Point3D(0.05, 38, 45),
                    new Point3D(-0.4, 19, 23)
                },
                new List<string> { "Spain", "Ukraine", "Poland", "Romania" });
            series3D2.Title = "<50 000 000";

            var data = new ObservableCollection<Series>();
            data.Add(series3D1);
            data.Add(series3D2);

            return data;
        }

        void cbShowScatter_Toggled(object sender, ToggledEventArgs e)
        {
            bubbleChart.ShowScatter = cbShowScatter.IsToggled;
            if (cbShowScatter.IsToggled && !cbShowLabels.IsToggled)
            {
                var scatterRenderer = bubbleChart.Plot.SeriesRenderers.Where(x => x is ScatterRenderer).FirstOrDefault() as ScatterRenderer;
                if (scatterRenderer != null)
                    scatterRenderer.ShowDataLabels ^= LabelKinds.InnerLabel;
            }
        }


        void cbShowLabels_Toggled(object sender, ToggledEventArgs e)
        {
            if (cbShowLabels.IsToggled)
            {
                bubbleChart.ShowDataLabels |= LabelKinds.InnerLabel;
                if (bubbleChart.ShowScatter)
                {
                    var scatterRenderer =  bubbleChart.Plot.SeriesRenderers.Where(x => x is ScatterRenderer).FirstOrDefault() as ScatterRenderer;
                    if (scatterRenderer != null)
                        scatterRenderer.ShowDataLabels |= LabelKinds.InnerLabel;
                }
            }
            else
            {
                bubbleChart.ShowDataLabels ^= LabelKinds.InnerLabel;
                if (bubbleChart.ShowScatter)
                {
                    var scatterRenderer = bubbleChart.Plot.SeriesRenderers.Where(x => x is ScatterRenderer).FirstOrDefault() as ScatterRenderer;
                    if (scatterRenderer != null)
                        scatterRenderer.ShowDataLabels ^= LabelKinds.InnerLabel;
                }
            }
            bubbleChart.Invalidate();
        }

        void cbLegend_Toggled(object sender, ToggledEventArgs e)
        {
            bubbleChart.ShowLegend = cbLegend.IsToggled;
        }
    }
}
