﻿using MindFusion.Charting;
using MindFusion.Drawing;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace FunnelChart
{
	// Learn more about making custom code visible in the Xamarin.Forms previewer
	// by visiting https://aka.ms/xamarinforms-previewer
	[DesignTimeVisible(false)]
	public partial class MainPage : ContentPage
	{
		public MainPage()
		{
			InitializeComponent();

			var values = new List<double> { 70, 60, 50, 30, 20, 15, 10, 4 };
			var labels = new List<string>{"Unqualified prospects", "Leads", "Initial communication",
				"Customer evaluation", "Negotiation", "Purchase order received", "Delivery", "Payment" };
			funnelChart.Series = new SimpleSeries(values, labels);

			var brushes = new List<Brush>
			{
					new SolidBrush(Color.FromHex("#669acc")),
					new SolidBrush(Color.FromHex("#616a7f")),
					new SolidBrush(Color.FromHex("#e0e9e9")),
					new SolidBrush(Color.FromHex("#114488")),
					new SolidBrush(Color.FromHex("#c0c0c0")),
					new SolidBrush(Color.FromHex("#ce0000")),
					new SolidBrush(Color.FromHex("#9caac6")),
					new SolidBrush(Color.FromHex("#333393"))
			};

			var strokes = new List<Brush> { new SolidBrush(Color.FromHex("#FFFFFF")) };

			var seriesBrushes = new List<List<Brush>> { brushes };
			var seriesStrokes = new List<List<Brush>> { strokes };
			
			var thicknesses = new List<double> { 6d };
			var seriesThicknesses = new List<List<double>> { thicknesses };

			funnelChart.Plot.SeriesStyle = new PerElementSeriesStyle(seriesBrushes, seriesStrokes, seriesThicknesses, null);
			funnelChart.Theme.DataLabelsFontSize = funnelChart.Theme.LegendTitleFontSize = 14;
			funnelChart.Theme.HighlightStroke = new SolidBrush(Color.FromHex("#ce0000"));
			funnelChart.Theme.DataLabelsBrush = new SolidBrush(Color.FromRgb(100, 100, 100));

			// create a custom legend renderer
			var seriesList = new ObservableCollection<Series>();
			var series = new SimpleSeries(null, null);
			series.Title = "Unqualified prospects=70K, 100%";
			seriesList.Add(series);
			series = new SimpleSeries(null, null);
			series.Title = "Leads=60K, 85.7%";
			seriesList.Add(series);
			series = new SimpleSeries(null, null);
			series.Title = "Initial communication=50K, 71.4%";
			seriesList.Add(series);
			series = new SimpleSeries(null, null);
			series.Title = "Customer evaluation=30K, 42.9%";
			seriesList.Add(series);
			series = new SimpleSeries(null, null);
			series.Title = "Negotiation=20K, 28.6%";
			seriesList.Add(series);
			series = new SimpleSeries(null, null);
			series.Title = "Purchase order received=15K, 21.4%";
			seriesList.Add(series);
			series = new SimpleSeries(null, null);
			series.Title = "Delivery=10K, 14.3%";
			seriesList.Add(series);
			series = new SimpleSeries(null, null);
			series.Title = "Payment=4K, 5.7%";
			seriesList.Add(series);

			var ren = new BarRenderer(seriesList);
			ren.Series = seriesList;
			ren.LabelFontSize = 12.0;
			ren.SeriesStyle = new PerSeriesStyle(brushes, null, null, null);

			funnelChart.LegendRenderer.Content.Clear();

			funnelChart.LegendRenderer.Background = funnelChart.Theme.DataLabelsBackground = new SolidBrush(Color.FromHex("#e0e9e9"));
			funnelChart.LegendRenderer.BorderStroke = funnelChart.Theme.DataLabelsBorderStroke = new SolidBrush(Color.FromHex("#c0c0c0"));
			funnelChart.LegendRenderer.TitleFontSize = 14.0;
			funnelChart.LegendRenderer.Title = "Distribution";
			funnelChart.LegendRenderer.ShowSeriesElements = false;
			funnelChart.LegendRenderer.Content.Add(ren);

			funnelChart.LegendVerticalAlignment = MindFusion.Charting.Components.LayoutAlignment.Far;

			funnelChart.StemWidth = 10 / 100f;
			funnelChart.ShowLegend = false;
		}

		private void chbShowLabels_Toggled(object sender, ToggledEventArgs e)
		{
			if (e.Value)
				funnelChart.ShowDataLabels |= LabelKinds.InnerLabel;
			else
				funnelChart.ShowDataLabels ^= LabelKinds.InnerLabel;

			funnelChart.Invalidate();
		}

		private void slStemWidth_ValueChanged(object sender, ValueChangedEventArgs e)
		{
			funnelChart.StemWidth = (float)e.NewValue / 100f;
			funnelChart.Invalidate();
		}

		private void slSegmentSpacing_ValueChanged(object sender, ValueChangedEventArgs e)
		{
			funnelChart.SegmentSpacing = (int)e.NewValue;
		}

		private void chbShowLegend_Toggled(object sender, ToggledEventArgs e)
		{
			funnelChart.ShowLegend = e.Value;
		}
	}
}
